<?

/*

sh4re - private and secure peer to peer file sharing system
===========================================================

Copyright (C) 2003 Vincent Negrier aka. sIX <six@aegis-corp.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

define("SH4_INC_DIR", ((strpos(strtoupper(PHP_OS), "WIN")!==false)?"C:\\sh4re\\inc":"/usr/lib/sh4re/inc"));
define("SH4_DEFAULT_CONTROL_PORT", 10102);

include SH4_INC_DIR.DIRECTORY_SEPARATOR."sh4re-core.php";

class sh4re_lib extends s2p_connection {

	var $shadok;
	var $cnx;

	var $host;
	var $port;
	var $net_key;
	var $connected = false;
	var $wait_pkts = array();
	var $s2p_agent = "";

	// Init
	
	function sh4re_lib($host="localhost", $port="10102", $net_key="") {

		$this->host = $host;
		$this->port = $port;
		$this->net_key = $net_key;

		$this->shadok = new SK_Server;
	
	}

	// Library functions
	
	function connect($timeout=5) {

		if (!$this->s2p_agent) $this->s2p_agent = "sh4re-lib/".SH4_VERSION;
		
		$sk_c_options = array(	"s2p_agent"				=> $this->s2p_agent,
								"lib"					=> &$this,
								"net_key"				=> $this->net_key);
		
		$t = time();

		$cid = $this->shadok->register_outgoing_connection("tcp://{$this->host}:{$this->port}", "sh4re_lib", $sk_c_options, $timeout, true);

		if (!$cid) return false;
		
		while ((!$this->connected) && (($t+$timeout)>time())) $this->shadok->run(1);
		
		if ($this->connected) $this->cnx =& $this->shadok->active_sessions[$cid];
		
		return $this->connected;

	}

	function say($msg) {

		$this->s2p_write(S2P_PKT_PUBMSG, $msg);
	
	}

	function say_to($node_id, $msg) {

		$this->s2p_write(S2P_PKT_PRIVMSG, sh4_serialize(array("node_id" => $node_id, "msg" => $msg)));
	
	}

	function net_connect() {

		$this->s2p_write(S2P_PKT_CONNECT);

	}

	function net_disconnect() {

		$this->s2p_write(S2P_PKT_DISCONNECT);

	}
	
	function shutdown($msg) {

		$this->s2p_write(S2P_PKT_SHUTDOWN, $msg);

	}

	function get_node_info() {

		$this->s2p_write(S2P_PKT_INFO_SRV);
		
		while ($this->connected && !$this->wait_pkts[S2P_PKT_INFO_SRV]) $this->shadok->run(-1);

		$ni_s = $this->wait_pkts[S2P_PKT_INFO_SRV][0];
		unset($this->wait_pkts[S2P_PKT_INFO_SRV]);

		$srv_info = sh4_unserialize($ni_s);

		return($srv_info);
	
	}
	
	function get_net_info($flags=SH4_NI_ALL) {

		$this->s2p_write(S2P_PKT_INFO_NET, $flags);
		
		while ($this->connected && !$this->wait_pkts[S2P_PKT_INFO_NET]) $this->shadok->run(-1);

		$ni_s = $this->wait_pkts[S2P_PKT_INFO_NET][0];
		unset($this->wait_pkts[S2P_PKT_INFO_NET]);

		$net_info = sh4_unserialize($ni_s);

		return($net_info);
	
	}

	function get_rights_list() {

		$this->s2p_write(S2P_PKT_RIGHT_LIST);
		
		while ($this->connected && !$this->wait_pkts[S2P_PKT_RIGHT_LIST]) $this->shadok->run(-1);

		$n_s = $this->wait_pkts[S2P_PKT_RIGHT_LIST][0];
		unset($this->wait_pkts[S2P_PKT_RIGHT_LIST]);

		$rights = sh4_unserialize($n_s);

		return($rights);

	}

	function access_grant($node_id, $rights) {

		$ni = $this->get_net_info(SH4_NI_NODES);
		foreach ($ni["nodes"] as $node) $ex_nodes[] = $node["node_id"];
		if (!in_array($node_id, $ex_nodes)) return false;
		
		$this->s2p_write(S2P_PKT_RIGHT_SET, sh4_serialize(array("node_id" => $node_id, "right" => $rights)));

		return true;
	
	}

	function access_revoke($node_id) {

		$ni = $this->get_net_info(SH4_NI_NODES);
		foreach ($ni["nodes"] as $node) $ex_nodes[] = $node["node_id"];
		if (!in_array($node_id, $ex_nodes)) return false;
		
		$this->s2p_write(S2P_PKT_RIGHT_SET, sh4_serialize(array("node_id" => $node_id, "right" => false)));

		return true;

	}

	function find_files($search_str, $local=false, $callback_wait_func=false) {

		if ($callback_wait_func && !is_callable($callback_wait_func)) $callback_wait_func = false;
		
		$a = array("local" => $local, "search" => $search_str);
		$s = sh4_serialize($a);

		$this->s2p_write(S2P_PKT_FIND, $s);
	
		while ($this->connected && !$this->wait_pkts[S2P_PKT_FIND]) {
			
			$this->shadok->run(-1);

			if ($callback_wait_func) $callback_wait_func();

		}

		$r = $this->wait_pkts[S2P_PKT_FIND][0];
		unset($this->wait_pkts[S2P_PKT_FIND]);

		$a = sh4_unserialize($r);

		return $a;
	
	}

	function get_files($file_hashes) {

		$s = sh4_serialize($file_hashes);
		$this->s2p_write(S2P_PKT_GET_HASHES, $s);

	}
	
	function get_node_status() {

		$this->s2p_write(S2P_PKT_STATUS);
		
		while ($this->connected && !$this->wait_pkts[S2P_PKT_STATUS]) $this->shadok->run(-1);

		$ni_s = $this->wait_pkts[S2P_PKT_STATUS][0];
		unset($this->wait_pkts[S2P_PKT_STATUS]);

		$srv_status = sh4_unserialize($ni_s);

		return($srv_status);
	
	}
	
	function rescan() {

		$this->s2p_write(S2P_PKT_RESCAN);
	
	}
	
	function cancel_download($dl_id) {

		$this->s2p_write(S2P_PKT_CANCEL, $dl_id);
	
	}

	function kill_sessions($sessids) {

		$this->s2p_write(S2P_PKT_KILL, sh4_serialize($sessids));
	
	}
	
	function get_errors() {

		$errs = $this->cnx->s2p_errors;
		$this->cnx->s2p_errors = array();

		return $errs;
	
	}

	// s2p_connetion callbacks

	function on_s2p_packet($type, $msg) {

		$this->options["lib"]->wait_pkts[$type][] = $msg;
			
	}
	
	function s2p_write($type, $msg=false) {

		if (is_object($this->options["lib"])) {

			parent::s2p_write($type, $msg);
		
		} else {

			$this->cnx->s2p_write($type, $msg);
			$this->shadok->empty_buffers();

		}

	}

	function on_s2p_connect() {

		$this->options["lib"]->connected = true;
	
	}

	function on_disconnect() {

		$this->options["lib"]->connected = false;
	
	}

}

?>
