#!/usr/local/bin/php
<?

/*

sh4re - private and secure peer to peer file sharing system
===========================================================

Copyright (C) 2003 Vincent Negrier aka. sIX <six@aegis-corp.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

define("SH4_DEFAULT_CONF_FILE", ((strpos(strtoupper(PHP_OS), "WIN")!==false)?"C:\\sh4re\\":"/etc/sh4re/")."sh4re.conf");
define("SH4_INC_DIR", ((strpos(strtoupper(PHP_OS), "WIN")!==false)?"C:\\sh4re\\inc":"/usr/lib/sh4re/inc"));

include SH4_INC_DIR.DIRECTORY_SEPARATOR."sh4re-lib.php";

// Misc

$stdin = fopen("php://stdin", "r");
stream_set_blocking($stdin, false);

define("SH4RED_HELP", <<<EOT
usage: ./sh4red.php [options]

-h --help             : this help page
-V --version          : show version number
-c --config=xx        : use specified config file
-a --address=##       : ip address to connect to
-p --port=##          : port to connect to
-k --key=secret       : use specified control key
-C --command="cmd"    : send command and exit
-q --quiet            : be quiet


EOT
);

// Help pages

define("HELP_PAGE_NONE", <<<EOT
help page not found

EOT
);


define("HELP_PAGE_DEFAULT", <<<EOT
available commands :

HELP         NETWORK      RIGHTS       NODEINFO     GRANT        REVOKE
SAY          MSG          FIND         LFIND        LIST         INFO
GET          HGET         CONNECT      DISCONNECT   STATUS       RESCAN
CANCEL       KILL         SHUTDOWN     QUIT

usage: HELP [COMMAND]

EOT
);

define("HELP_PAGE_NETWORK", <<<EOT
show network informations including online nodes and total amount of sh4red
content on the network

usage: NETWORK

EOT
);

define("HELP_PAGE_RIGHTS", <<<EOT
show your rights and the one you granted on each node of the network

usage: RIGHTS

EOT
);

define("HELP_PAGE_NODEINFO", <<<EOT
get detailed informations about the specified node

usage: NODEINFO <NODEID>

EOT
);

define("HELP_PAGE_GRANT", <<<EOT
grant access to the specified node to your content

usage: GRANT <NODEID> [<FLAGS>]

FLAGS are :

s : right to search stuff on your node
d : right to download stuff on your node
u : right to upload stuff on your node

if no flags are specified, the default "sd" is used

examples : GRANT 4
           GRANT 1 sd
           GRANT 6 sdu

EOT
);

define("HELP_PAGE_REVOKE", <<<EOT
revoke all access flags from the specified node

usage: REVOKE <NODEID>

EOT
);

define("HELP_PAGE_SAY", <<<EOT
send a public text message to the network

usage: SAY <your text>

EOT
);

define("HELP_PAGE_MSG", <<<EOT
send a private text message to a network node

usage: MSG <NODEID> <your text>

EOT
);

define("HELP_PAGE_FIND", <<<EOT
find content on the network

usage: FIND <search-string>

the <search-string> is composed of space delimited tokens being either
<search-text> or <search-expression>s

<search-text> is either a word of text or a " or ' enclosed string that may
contain spaces

<search-expression> is a string of form <criterium><operator><value>[<suffix>]

<criterium> must be one of the following :
TYPE      AGE       SIZE      SAMEDIR   META

<operator> must be one of the following :
=         <         >

<value> and <suffix> depend on the criterium :

for TYPE, <value> must be "Audio", "Video" or "Other"
for AGE, <value> must be a number, and <suffix> must be one of the following:
d (for days), w (for weeks), m (for months), y (for years)
for SIZE, <value> must be a number, and <suffix> must be one of the following:
<none> (for bytes), k (for KBytes), m (for MBytes), g (for GBytes)
for SAMEDIR, <value> must be a valid file hash, or a "$" prefixed file id
for META, <value> must be set to "n" and meta-info search will be disabled

examples: FIND "man with no name" type=audio
          FIND age<10d
          FIND samedir=$1
          FIND futurama type=video size>100M
          FIND wiliam shatner rocketman

EOT
);

define("HELP_PAGE_LFIND", <<<EOT
find content on the local node (see HELP FIND for details)

usage: LFIND <search-string>

EOT
);

define("HELP_PAGE_LIST", <<<EOT
show results list from last search

usage: LIST

EOT
);

define("HELP_PAGE_INFO", <<<EOT
display extended informations including meta-info about the specified item

usage: INFO <FILEID>

EOT
);

define("HELP_PAGE_GET", <<<EOT
get file(s) from the network

usage: GET <files-list>

<files-list> is either a single <range>, or a comma serparated list of <range>

<range> can be of different forms :
"6" means file with ID = 6
"12-18" means files with ID ranging from 12 to 18 inclusive
"50-" means all files from the one with id 50 inclusive
"-10" means all files from 1 to the one with id 10 inclusive
"*" means all files in the previously acquired find list

examples: GET 3
          GET 3,4
          GET -8,10-
          GET 3-10
          GET 3,5-11,15,20-
          GET *

EOT
);

define("HELP_PAGE_HGET", <<<EOT
get file(s) from the network by hash

usage: HGET <hashes-list>

<hashes-list> is either a single hash, or a comma separated list of hashes

example: HGET fcef1cf509b5ba4324ab9eeb775122c0

EOT
);

define("HELP_PAGE_STATUS", <<<EOT
get current node status including pending file transfers and active control
connections

usage: STATUS

EOT
);

define("HELP_PAGE_CONNECT", <<<EOT
connect to the network

usage: CONNECT

EOT
);

define("HELP_PAGE_DISCONNECT", <<<EOT
disconnect from the network

usage: DISCONNECT

EOT
);

define("HELP_PAGE_RESCAN", <<<EOT
rescan network for download list availability

usage: RESCAN

EOT
);

define("HELP_PAGE_CANCEL", <<<EOT
cancel a download and erase incomplete file

usage: CANCEL <download-connection-id>

EOT
);

define("HELP_PAGE_KILL", <<<EOT
kill a connetion (download, upload, or control)

usage: KILL <any-connection-id>

EOT
);


define("HELP_PAGE_SHUTDOWN", <<<EOT
disconnect the node from the network and shutdown the daemon

usage: SHUTDOWN [reason text]

EOT
);

// Display funcs

function net_info_s($net_info) {

	$tot = $net_info["network"]["network_name"]." : ".count($net_info["active_nodes"])."/".count($net_info["nodes"])." node(s) online/total";
	
	$ret .= nodes_list_s($net_info["nodes"], true, false, array_keys($net_info["active_nodes"]), $net_info["active_nodes"]);
	$ret .= "------------------------------------------------ ------------- -----------\n";
	$ret .= str_pad($tot, 49).str_pad(number_format($net_info["network"]["files_total"]), 12, " ", STR_PAD_LEFT)." ".str_pad(number_format($net_info["network"]["mbytes_total"]/1024, 2), 12, " ", STR_PAD_LEFT)."\n";

	return $ret;

}

function nodes_list_s($net_info_nodes, $r_list=false, $num_listed=false, $prefix_online=array(), $act_list=false) {

	ksort($net_info_nodes);
	
	if (is_array($r_list)) {

		$add_1 = " Local Flags  Remote Flags";
		$add_2 = " ------------ ------------";
	
	} else if ($r_list === true) {

		$add_1 = "        Files    GigaBytes";
		$add_2 = " ------------ ------------";
	
	}
	
	$ret  = "NodeID User     Node Name                       ".$add_1."\n";
	$ret .= "------ -------- --------------------------------".$add_2."\n";

	$nb_nodes = count($net_info_nodes);
	
	if ($net_info_nodes) foreach ($net_info_nodes as $k=>$node) {

		$ret .= in_array($node["node_id"], $prefix_online) ? "*" : " ";
		$ret .= str_pad($node["node_id"], 5, " ", STR_PAD_LEFT)." ";
		$ret .= str_pad($node["node_short_name"], 8)." ";
		$ret .= str_pad($node["node_name"], 32);
		
		if (is_array($r_list)) {
		
			$ret .= " ";
			$ret .= str_pad($r_list["from"][$node["node_id"]]["node_rights"], 12, " ", STR_PAD_BOTH)." ";
			$ret .= str_pad($r_list["to"][$node["node_id"]]["node_rights"], 12, " ", STR_PAD_BOTH);

		} else if ($r_list === true) {

			$n_files = $act_list[$k]["node_files_a"] + $act_list[$k]["node_files_v"] + $act_list[$k]["node_files_o"];
			$n_mbytes = (int) ($act_list[$k]["node_mbytes_a"] + $act_list[$k]["node_mbytes_v"] + $act_list[$k]["node_mbytes_o"]);

			$ret .= " ";

			if (($n_files == 0) && ($n_mbytes == 0)) {
				
				$n_files = $n_mbytes = "-";

			} else {

				$n_files = number_format($n_files);
				$n_mbytes = number_format($n_mbytes/1024, 2);
			
			}
			
			$ret .= str_pad($n_files, 12, " ", STR_PAD_LEFT)." ";
			$ret .= str_pad($n_mbytes, 12, " ", STR_PAD_LEFT);
		
		}

		$ret .= "\n";

	} else {

		$ret .= "not connected\n";
	
	}

	if ($num_listed) $ret .= "\n".$nb_nodes." node".($nb_nodes>1?"s":"")." listed\n";

	return $ret;

}

function echo_find_wait() {

	static $cnt;
	static $chr;

	if (!--$cnt) {
	
		$chr = ($chr + 1)%4;
		$chrs = "|/-\\";

		echo chr(13)."Searching ... ".$chrs{$chr};

		$cnt = 64;
	
	}

}

function echo_find_end() {

	echo chr(13)."               ".chr(13);

}

function echo_find_result($r) {

	if ($r) {

		echo "FID# File Name                             Date                Size Type  Avail\n";
		echo "---- ------------------------------------- ---------- ------------- ----- -----\n";
		 
		foreach ($r as $k=>$f) {

			$l =  str_pad($k+1, 4, " ", STR_PAD_LEFT)." ";
			$l .= str_pad(substr($f["file_name"], 0, 37), 37)." ";
			$l .= substr($f["file_date"], 0, 10)." ";
			$l .= str_pad(number_format($f["file_size"]), 13, " ", STR_PAD_LEFT)." ";
			$l .= str_pad($f["file_type"], 5)." ";
			$l .= str_pad($f["avail"], 5, " ", STR_PAD_BOTH);

			echo $l."\n";

			++$n_files;
			$n_bytes += $f["file_size"];

		}

		echo "-------------------------------------------------------------------------------\n";

	}
	
	echo "Found ".sh4_size_format($n_bytes)." in ".(int)$n_files." file(s)\n";

}

function echo_file_info($file) {

	echo "File Info\n-------------------------------------------------------------------------------\n";

	echo "Name : ".$file["file_name"]."\n";
	if ($file["file_directory"]) echo "Dir  : ".$file["file_directory"]."\n";
	echo "Date : ".$file["file_date"]."\n";
	echo "Size : ".number_format($file["file_size"])." bytes\n";
	echo "Hash : ".$file["file_hash"]."\n";
	echo "Type : ".$file["file_type"]."\n";
	echo "#Avl : available at ".$file["avail"]." node(s)\n";
	
	echo "-------------------------------------------------------------------------------\n";
	
	foreach (array_keys($file) as $k) if ($maxl < strlen($k)) $maxl = strlen($k);
	foreach ($file as $k=>$v) if ((substr($k, 0, 5) != "file_") && ($k != "avail")) echo str_pad($k, $maxl)." : ".$v."\n";

	echo "-------------------------------------------------------------------------------\n";

}

function echo_status($status) {

	echo "-- Node Status ----------------------------------------------------------------\n";
	echo "Network connection : ".($status["master_cnx"] ? ("Online (".strtok($status["master_cnx"], "#").")") : "Offline")."\n";
	echo "Node uptime        : ".ts_to_string($status["current_time"] - $status["start_time"])."\n";
	echo "Xfer connections   : ".number_format($status["xfer_cnx_total"])." (".number_format($status["xfer_cnx_errors"])." errors)\n";
	echo "Xfer bytes in/out  : ".sh4_size_format($status["xfer_rcv_bytes"])." / ".sh4_size_format($status["xfer_sent_bytes"])." (total ".sh4_size_format($status["xfer_rcv_bytes"] + $status["xfer_sent_bytes"]).")\n";

	if ($status["control"]) {
	
		echo "-- Control Connections --------------------------------------------------------\n";
		
		$cli = 0;
		
		foreach ($status["control"] as $k=>$ctl) {

			$GLOBALS["controls_index"][++$cli] = array($k);
			echo str_pad("C".$cli, 5)." ".str_pad($ctl["remote_ip"].":".$ctl["remote_port"], 21)." ".str_pad(date("M d H:i", $ctl["accept_time"]), 12)." (idle ".ts_to_string($ctl["idle_time"]).")\n";
		
		}
	
	}

	$GLOBALS["downloads_index"] = array();
	
	if ($status["downloads"]) {
	
		$u_s = " Downloads ";
		
		echo "--".str_pad($u_s, 77, "-")."\n";

		$dli = 0;
		
		foreach ($status["downloads"] as $k=>$dl) {

			$fn = $dl["file_name"]
			or $fn = $k;

			if (strlen($fn) > 21) $fn = "+".substr($fn, -20);

			$db_s = $dl["blocks_25c"]
			or $db_s="-------------------------";

			$GLOBALS["downloads_index"][++$dli] = $k;
			$GLOBALS["downloads_cnx_index"][$dli] = array();
			echo str_pad("D".$dli, 5)." ".str_pad($fn, 21)." (".str_pad(number_format($dl["file_size"]), 13, " ", STR_PAD_LEFT)." bytes)   [".$db_s."]\n";

			if ($dl["connections"]) foreach ($dl["connections"] as $ck=>$cnx) {
		
				$GLOBALS["downloads_cnx_index"][$dli][] = $ck;
				echo str_pad("D".$dli, 5)." ".str_pad($cnx["node_name"], 21)." ".str_pad($cnx["ip_addr"].":".$cnx["ip_port"], 21)."   ".str_pad(ts_to_string(time() - $cnx["connect_time"]), 15)." ".str_pad((sh4_size_format($cnx["rate_in"])."/s"), 11, " ", STR_PAD_LEFT)."\n";

			}
		
		}

	}

	$GLOBALS["uploads_index"] = array();
	
	if ($status["uploads"]) {
	
		$u_s = " Uploads (".$status["slots_up_used"]."/".$status["slots_up"].") ";
		
		echo "--".str_pad($u_s, 77, "-")."\n";

		$uli = 0;
		
		foreach ($status["uploads"] as $k=>$ul) {

			$fn = $ul["file_name"]
			or $fn = $k;

			if (strlen($fn) > 21) $fn = "+".substr($fn, -20);
			
			$db_s = $ul["blocks_25c"]
			or $db_s="-------------------------";
			
			$GLOBALS["uploads_index"][++$uli] = array($k);
			echo str_pad("U".$uli, 5)." ".str_pad($fn, 21)." (".str_pad(number_format($ul["file_size"]), 13, " ", STR_PAD_LEFT)." bytes)   [".$db_s."]\n";
		
			echo str_pad("U".$uli, 5)." ".str_pad($ul["node_name"], 21)." ".str_pad($ul["ip_addr"].":".$ul["ip_port"], 21)."   ".str_pad(ts_to_string(time() - $ul["accept_time"]), 15)." ".str_pad((sh4_size_format($ul["rate_out"])."/s"), 11, " ", STR_PAD_LEFT)."\n";
		
		}
	
	}
	
	echo "-------------------------------------------------------------------------------\n";

}

function node_info_s($an) {

	$ret = "-- Node MOTD ------------------------------------------------------------------\n";
	$ret .= $an["node_motd"]."\n";
	$ret .= "-- Node Info ------------------------------------------------------------------\n";
	$ret .="ID#    : ".($an["node_netop"] ? "@" : "").$an["node_id"]."\n";
	$ret .="User   : ".$an["node_short_name"]."\n";
	$ret .="Name   : ".$an["node_name"]."\n";
	$ret .="URL    : ".$an["node_url"]."\n";
	$ret .="Agent  : ".$an["s2p_agent"]."\n";
	$ret .="Files  : ".number_format($an["node_files_a"] + $an["node_files_v"] + $an["node_files_o"])." (V:".number_format($an["node_files_v"])." A:".number_format($an["node_files_a"])." O:".number_format($an["node_files_o"]).")\n";
	$ret .="MBytes : ".number_format($an["node_mbytes_a"] + $an["node_mbytes_v"] + $an["node_mbytes_o"])." (V:".number_format($an["node_mbytes_v"])." A:".number_format($an["node_mbytes_a"])." O:".number_format($an["node_mbytes_o"]).")\n";
	$ret .= "-------------------------------------------------------------------------------\n";

	return $ret;
}

function parse_get_arg_to_hashes($s, $result) {

	$hashes = array();
	
	foreach (sh4_parse_range($s, count($result) - 1) as $a) if ((is_array($result[$a - 1])) && (!in_array($result[$a - 1]["file_hash"], $hashes))) $hashes[$result[$a - 1]["file_hash"]] = $result[$a - 1]["file_name"];
	
	return $hashes;

}

function ts_to_string($timestamp) {

	$up_d=(int)($timestamp/86400);
	$up_h=((int)($timestamp/3600))%24;
	$up_m=((int)($timestamp/60))%60;
	$up_s=$timestamp%60;

	$tstr=($up_d?($up_d."d, "):"").(($up_h||$up_d)?($up_h."h, "):"").(($up_m||$up_h||$up_d)?($up_m."m, "):"").$up_s."s";

	return ($tstr);

}

// Init

$cmdline = sh4_parse_cmdline_args(array(	"-h" => "--help",
											"-V" => "--version",
											"-c" => "--config",
											"-a" => "--address",
											"-p" => "--port",
											"-k" => "--key",
											"-C" => "--command",
											"-q" => "--quiet"));

if ($cmdline["-V"]) die(SH4_VERSION."\n");
if ($cmdline["-h"]) die(SH4RED_HELP);

$host = $cmdline["-a"] or
$host = "127.0.0.1";

$port = $cmdline["-p"] or
$port = 10102;

$sh_cmd = $cmdline["-C"];

if (!$cmdline["-q"]) echo "sh4re-cli/".SH4_VERSION." (c) 2003 by sIX / aEGiS <six@aegis-corp.org>\n";

$conf_f = $cmdline["-c"]
or $conf_f = SH4_DEFAULT_CONF_FILE;

if ($conf = sh4_parse_config($conf_f)) {

	$ctl_key = $conf["controlkey"][0];

} else if ($cmdline["-c"]) {

	errexit("FATAL: unable to read conf file '$conf_f'");

}

if ($cmdline["-k"]) $ctl_key = $cmdline["-k"];

$slib = new sh4re_lib($host, $port, $ctl_key);

$slib->s2p_agent = "sh4re-cli/".SH4_VERSION;

if (($slib->connect()) && ($node = $slib->get_node_info())) {

	if (!$cmdline["-q"]) echo "connected to ".$node["node_name"]." at $host:$port (".$slib->cnx->remote_s2p_agent.")\n";

} else {

	die("unable to connect to $host:$port\n");

}

if (!$sh_cmd) echo "\nsh4re> ";

while (!$exit) {

	if ($sh_cmd) {

		$exit = true;
		$s = $sh_cmd;
	
	} else {
	
		$s = fgets($stdin, 4096);

	}
	
	if ($s) {
	
		$s = rtrim($s, "\n\r");		
		$cmd = trim(strtoupper(strtok($s, " ")));
		$arg = strtok("");

		switch ($cmd) {

			case "HELP":
			
			if (!$arg2 = trim($arg)) $arg2 = "DEFAULT";
			if (strtoupper($arg2) == "HELP") $arg2 = "DEFAULT";

			if (!defined($cn = ("HELP_PAGE_".strtoupper(trim($arg2))))) $cn = "HELP_PAGE_NONE";
			
			echo constant($cn);
			
			break;
			
			case "":
			$slib->shadok->run(-1);
			break;
			
			case "NETWORK":
			if ($net_info = $slib->get_net_info()) echo net_info_s($net_info)."\n";
			break;

			case "NODEINFO":

			if ($act_n = $slib->get_net_info(SH4_NI_ACTIVE_NODES)) {

				if (!in_array($arg, array_keys($act_n["active_nodes"]))) {

					echo "node $arg is not online\n";	
				
				} else {

					echo node_info_s($act_n["active_nodes"][$arg]);

				}

			}

			break;
			
			case "RIGHTS":
			if (($net_info = $slib->get_net_info(SH4_NI_NODES | SH4_NI_ACTIVE_NODES)) && is_array($r_list = $slib->get_rights_list())) echo nodes_list_s($net_info["nodes"], $r_list, true, array_keys($net_info["active_nodes"]))."\n";
			break;
			
			case "GRANT":

			$node = strtok($arg, " ");

			$rights = strtok("") or
			$rights = "sd";
			
			if ($slib->access_grant($node, $rights)) {

				echo "granted $arg\n";

			} else {

				echo "grant $arg failed\n";
			
			}
			break;

			case "REVOKE":
			if ($slib->access_revoke($arg)) {

				echo "revoked $arg\n";

			} else {

				echo "revoke $arg failed\n";
			
			}
			break;
			
			case "SAY":
			$slib->say($arg);
			break;

			case "MSG":
			$nto = strtok($arg, " ");
			$msg = strtok("");
			$slib->say_to($nto, $msg);
			echo "said '$msg' to node #$nto\n";
			break;
			
			case "LFIND":
			case "FIND":
			
			if (strpos($arg, '$') !== false) {

				$r = array();
				if ($find_result) foreach ($find_result as $k=>$v) $r['$'.($k + 1)] = $v["file_hash"];
				$arg = strtr($arg, $r);
				unset($r);

			}

			$find_result = $slib->find_files($arg, ($cmd == "LFIND"), ($cmdline["-q"] ? false : "echo_find_wait"));
			
			if (!$cmdline["-q"]) echo_find_end();
			
			echo_find_result($find_result);
			
			break;

			case "LIST":
			echo_find_result($find_result);
			break;
			
			case "INFO":
			if ($f = $find_result[$arg - 1]) {

				echo_file_info($f);

			} else {

				echo "item not found\n";
			
			}
			break;
			
			case "GET":

			if ($hlist = parse_get_arg_to_hashes($arg, $find_result)) {
				
				$slib->get_files($hlist);

			}

			break;

			case "HGET":
			
			if (($arg = trim($arg)) && (strlen($arg) == SH4_FILE_HASH_LEN)) $slib->get_files(array($arg => null));

			break;
			
			case "STATUS":
			echo_status($slib->get_node_status());
			break;
			
			case "QUIT":
			case "EXIT":
			$exit = true;
			echo "bye!\n";
			break;

			case "SHUTDOWN":
			if ($arg) $msg = $arg;
			else $msg = "requested";
			echo "shutdown in progress\n";
			$slib->shutdown($msg);
			break;

			case "CONNECT":
			$slib->net_connect();
			break;

			case "DISCONNECT":
			$slib->net_disconnect();
			break;
			
			case "RESCAN":
			$slib->rescan();
			break;

			case "CANCEL":
			
			if (strtoupper($arg{0}) !== "D") {

				echo "can only cancel downloads\n";
				break;

			}

			$dl_id = (int)substr($arg, 1);

			if (!in_array($dl_id, array_keys($GLOBALS["downloads_index"]))) {

				echo "not an active download\n";
				break;

			}

			$slib->cancel_download($GLOBALS["downloads_index"][$dl_id]);

			break;

			case "KILL":

			$kill_t = strtoupper($arg{0});
			$kill_id = (int)substr($arg, 1);
			$kill_ids = array();

			switch ($kill_t) {

				case "D":
				$kill_ids = $GLOBALS["downloads_cnx_index"][$kill_id];
				break;
				
				case "U":
				$kill_ids = $GLOBALS["uploads_index"][$kill_id];
				break;
				
				case "C":
				$kill_ids = $GLOBALS["controls_index"][$kill_id];
				break;

				default:
				echo "ask someone else for the job\n";
				break 2;

			}

			if ($kill_ids) $slib->kill_sessions($kill_ids);
			
			break;

			default:
			echo "$cmd is not a valid command, try 'help'\n";
			break;
		
		}

		if (!$exit) echo "sh4re> ";
	
	}
	
	if (!$slib->connected) {
		
		echo "fatal: lost connection to sh4red\n";
		$exit = true;
	
	}

	// empty MASTER_NOTICE bucket
	
	if ($slib->wait_pkts[S2P_PKT_MASTER_NOTICE]) {
		
		foreach ($slib->wait_pkts[S2P_PKT_MASTER_NOTICE] as $msg) echo "[".date("H:i")."] * ".$msg."\n";

		$slib->wait_pkts[S2P_PKT_MASTER_NOTICE] = array();

	}
	
	// empty MASTER_FATAL bucket
	
	if ($slib->wait_pkts[S2P_PKT_MASTER_FATAL]) {
		
		foreach ($slib->wait_pkts[S2P_PKT_MASTER_FATAL] as $msg) echo "[".date("H:i")."] !!! ".$msg."\n";

		$slib->wait_pkts[S2P_PKT_MASTER_FATAL] = array();

	}
	
	// empty NOTICE bucket
	
	if ($slib->wait_pkts[S2P_PKT_NOTICE]) {
		
		foreach ($slib->wait_pkts[S2P_PKT_NOTICE] as $msg) echo "[".date("H:i")."] * ".$msg."\n";

		$slib->wait_pkts[S2P_PKT_NOTICE] = array();

	}
	
	// empty PUBMSG bucket
	
	if ($slib->wait_pkts[S2P_PKT_PUBMSG]) {
		
		foreach ($slib->wait_pkts[S2P_PKT_PUBMSG] as $msg) {

			$u = sh4_unserialize($msg);
			
			echo "[".date("H:i")."] ".$u["name"]."> ".$u["msg"]."\n";

		}
	
		$slib->wait_pkts[S2P_PKT_PUBMSG] = array();

	}

	// empty PRIVMSG bucket
	
	if ($slib->wait_pkts[S2P_PKT_PRIVMSG]) {
		
		foreach ($slib->wait_pkts[S2P_PKT_PRIVMSG] as $msg) {

			$u = sh4_unserialize($msg);
			
			echo "[".date("H:i")."] MSG:".$u["name"]."(".$u["node_id"].")> ".$u["msg"]."\n";

		}
	
		$slib->wait_pkts[S2P_PKT_PRIVMSG] = array();

	}
	
	$slib->shadok->run(-1);

}

if (!$sh_cmd) echo "\n";

?>
